/* updated 08/13
*      Aaron W.
*      http://letsmakerobots.com/node/2935
*      
*      dual sharp IR mintvelt tracker.
*      trying to add encoders again
*/
//servo ranges from about 40(0x28) to 120(0x78) center is 0x55.

#include "Wire.h"
#include <LCDI2C4Bit.h>

// These lines are neccessary for the wireless rf library
#include <VirtualWire.h>
#undef int
#undef abs
#undef double
#undef float
#undef round

#define DEBUG 1

#define RTC_I2C_ADR 0x51        // I2C device addresses
#define LCD_I2C_ADR 0x6D
#define EEPROM_I2C_ADR 0x53
#define SLAVE_I2C_ADR 4    // this devices address

#define FORWARD "0CD 14"  // forward
#define BACKWARD "0CD 28"  //backward
#define STOP "0CD 3C"  //stop
#define SPIN_LEFT "0CD 18"  //turn on spot left
#define SPIN_RIGHT "0CD 24"  //turn on spot right
#define CURVE_LEFT "0CD 10"  //turn left
#define CURVE_RIGHT "0CD 04"  //turn right
#define MOTOR_PWM_LEFT 5  //PWM pins
#define MOTOR_PWM_RIGHT 6

// pins for the motor encoders
#define MOTOR_ENC_LEFT 2
#define MOTOR_ENC_RIGHT 3

// the max and min decimal positions that should be sent to the servos.
#define SER_LOW_LIMIT 40
#define SER_HIGH_LIMIT 120

// pins for the maxbotics US sensor
#define US_PULSE 7   
#define US_TRIGGER 8

// pins for sharp IR
#define RANGE_IR_PIN_LEFT 2
#define RANGE_IR_PIN_RIGHT 3

// some range constants
#define RANGE_DANGER 15
#define RANGE_CAUTION 30

// full scan distance array.
int sonarRange[10];

LCDI2C4Bit lcd = LCDI2C4Bit(LCD_I2C_ADR,2,16);        //LCD definition

uint8_t lcdSmile[8] = {0x00,0x0A,0x0A,0x00,0x11,0x11,0x0E,0x00}; // smiley face
uint8_t lcdReturn[8] = {0x00,0x01,0x01,0x05,0x0D,0x1F,0x0C,0x04}; // return icon
uint8_t lcdBack[8] = {0x04,0x06,0x1F,0x16,0x14,0x10,0x10,0x00}; // back icon
uint8_t lcdRight[8] = {0x00,0x00,0x04,0x06,0x1F,0x06,0x04,0x00}; // right icon
uint8_t lcdLeft[8] = {0x00,0x00,0x04,0x0C,0x1F,0x0C,0x04,0x00}; // left icon

int speedLeft = 254;  // pwm levels for the motor driver. The right side motor is slightly faster
int speedRight = 244;

// encoder counts
volatile unsigned int encLeftCount = 0;
volatile unsigned int encRightCount = 0;

char buffer[10];

typedef struct {
  byte command;
  char delim;
  char buffer [5];   //don't forget to account for the terminator.
} packet_t;

/*
*  Encoder interrupt functions.
*  Not complete.
*/
void encLeft()
{
  encLeftCount++;
}

void encRight()
{
  encRightCount++;
}

/*
*  Wireless RF wrapper functions.
*  Compile packet to send. Integer data only.
*/
void rfPacketOut(char commandChar,int data)
{
  packet_t packet;
  packet.command = commandChar;
  packet.delim = '.';
  itoa(data, packet.buffer, 10);
  vw_send((uint8_t *)&packet, sizeof(packet));
   
}

/*
*  LED control for the buttons. Actually controlled by the I2C expander for the LCD
*  1 = on, 0 = off
*/
void buttonLights(int onOff)
{
  int bitMask = B10000000;
  Wire.beginTransmission(LCD_I2C_ADR);
  if(onOff == 0) Wire.send(0xFF);
  else if(onOff == 1) Wire.send(0x00);
  Wire.endTransmission();
}

/*
*  RTC functions
*
*/
// Convert normal decimal numbers to binary coded decimal
byte decToBcd(byte val)
{
  return ( (val/10*16) + (val%10) );
}

// Convert binary coded decimal to normal decimal numbers
byte bcdToDec(byte val)
{
  return ( (val/16*10) + (val%16) );
}

void setDateRTC(byte second,        // 0-59
                   byte minute,        // 0-59
                   byte hour,          // 1-23
                   byte dayOfMonth,     // 1-31
                   byte dayOfWeek,    // 1-7
                   byte month,         // 1-12
                   byte year)          // 0-99
{
   Wire.beginTransmission(RTC_I2C_ADR);
   Wire.send(0);
   Wire.send(0x00);
   Wire.send(0x00);
   Wire.send(decToBcd(second));    // 0 to bit 7 starts the clock
   Wire.send(decToBcd(minute));
   Wire.send(decToBcd(hour));      // If you want 12 hour am/pm you need to set
                                   // bit 6 (also need to change readDateRTC)
   Wire.send(decToBcd(dayOfMonth));
   Wire.send(decToBcd(dayOfWeek));
   Wire.send(decToBcd(month));
   Wire.send(decToBcd(year));
   Wire.endTransmission();
}

void getDateRTC(byte *second,
          byte *minute,
          byte *hour,
          byte *dayOfMonth,
          byte *dayOfWeek,
          byte *month,
          byte *year)
{
  
  // Reset the register pointer
  Wire.beginTransmission(RTC_I2C_ADR);
  Wire.send(0);
  Wire.endTransmission();
  Wire.requestFrom(RTC_I2C_ADR, 9);

  // A few of these need masks because certain bits are control bits
  *second     = bcdToDec(Wire.receive());
  *second     = bcdToDec(Wire.receive());
  *second     = bcdToDec(Wire.receive() & 0x7f);
  *minute     = bcdToDec(Wire.receive() & 0x7f);
  *hour       = bcdToDec(Wire.receive() & 0x3f);  // Need to change this if 12 hour am/pm
  *dayOfMonth  = bcdToDec(Wire.receive() & 0x3f);
  *dayOfWeek = bcdToDec(Wire.receive() & 0x07);
  *month      = bcdToDec(Wire.receive() & 0x1f);
  *year       = bcdToDec(Wire.receive());
}

void getAlarmRTC(byte *minute,
          byte *hour,
          byte *dayOfMonth,
          byte *dayOfWeek)
{
  
  // Reset the register pointer
  Wire.beginTransmission(RTC_I2C_ADR);
  Wire.send(0x09);
  Wire.endTransmission();
  Wire.requestFrom(RTC_I2C_ADR, 4);

  // A few of these need masks because certain bits are control bits
  *minute     = bcdToDec(Wire.receive() & 0x7f);
  *hour       = bcdToDec(Wire.receive() & 0x3f);  // Need to change this if 12 hour am/pm
  *dayOfMonth  = bcdToDec(Wire.receive() & 0x3f);
  *dayOfWeek = bcdToDec(Wire.receive() & 0x07);
}

void setAlarmRTC(byte minute, byte hour, byte day, byte weekday)
{
  Wire.beginTransmission(RTC_I2C_ADR);
  Wire.send(0x09);
  Wire.send(decToBcd(minute));
  Wire.send(decToBcd(hour));
  Wire.send(decToBcd(day));
  Wire.send(decToBcd(weekday));
  Wire.endTransmission();
  
}

/*
*  EEPROM functions.
*
*/ 
void EepromRead(int theDeviceAddress, unsigned int theMemoryAddress, int theByteCount, byte* theByteArray) {
  for (int theByteIndex = 0; theByteIndex < theByteCount; theByteIndex++) {
    Wire.beginTransmission(theDeviceAddress);
    Wire.send((byte)((theMemoryAddress + theByteIndex) >> 8));
    Wire.send((byte)((theMemoryAddress + theByteIndex) >> 0));
    Wire.endTransmission();
    delay(5);
    Wire.requestFrom(theDeviceAddress, sizeof(byte));
    theByteArray[theByteIndex] = Wire.receive();
  }
}

byte EepromReadByte(int theDeviceAddress, unsigned int theMemoryAddress) {
  byte theByteArray[sizeof(byte)];
  EepromRead(theDeviceAddress, theMemoryAddress, sizeof(byte), theByteArray);
  return (byte)(((theByteArray[0] << 0)));
}

int EepromReadInt(int theDeviceAddress, unsigned int theMemoryAddress) {
  byte theByteArray[sizeof(int)];
  EepromRead(theDeviceAddress, theMemoryAddress, sizeof(int), theByteArray);
  return (int)(((theByteArray[0] << 8)) | (int)((theByteArray[1] << 0)));
}

void EepromWrite(int theDeviceAddress, unsigned int theMemoryAddress, int theByteCount, byte* theByteArray) {
  for (int theByteIndex = 0; theByteIndex < theByteCount; theByteIndex++) {
    Wire.beginTransmission(theDeviceAddress);
    Wire.send((byte)((theMemoryAddress + theByteIndex) >> 8));
    Wire.send((byte)((theMemoryAddress + theByteIndex) >> 0));
    Wire.send(theByteArray[theByteIndex]);
    Wire.endTransmission();
    delay(5);
  }
}

void EepromWriteByte(int theDeviceAddress, unsigned int theMemoryAddress, byte theByte) {
  byte theByteArray[sizeof(byte)] = {(byte)(theByte >> 0)};
  EepromWrite(theDeviceAddress, theMemoryAddress, sizeof(byte), theByteArray);
}

void EepromWriteInt(int theDeviceAddress, unsigned int theMemoryAddress, int theInt) {
  byte theByteArray[sizeof(int)] = {(byte)(theInt >> 8), (byte)(theInt >> 0)};
  EepromWrite(theDeviceAddress, theMemoryAddress, sizeof(int), theByteArray);
}

/*
*  Movement routines. Time is expressed in milliseconds.
*  Moves forward until time out. If time = 0 start motion and return to caller
*/
int mGo(char* _direction, int _time, boolean headLocked)
{
  unsigned long startTime = millis();
  unsigned long durationMillis = _time;

  Serial.println(_direction);  // starts the motion
  if(_time == 0) return true;  
  
  if(headLocked == 0)  // head is NOT locked so we can fool with it before running the loop
  {
    if((_direction == FORWARD)||(_direction == STOP))
    {
      headReset();
    }
    else if(_direction == BACKWARD)
    {
      
    }
    else if(_direction == SPIN_LEFT)
    {
      servoPos(0,170);
    }
    else if(_direction == SPIN_RIGHT)
    {
      servoPos(0,10);
    }
    else if(_direction == CURVE_LEFT)
    {
      servoPos(0,130);
    }
    else if(_direction == CURVE_RIGHT)
    {
      servoPos(0,50);
    }
    
    servoPos(0,90);
  }
  while(startTime+durationMillis >= millis())  // loop endlessly until we time out
  {  
  }

  Serial.println(STOP);
  return true;
}

/*
*  Glances down to check low lying obstacles.
*  Returns true if either a cliff or an obstacle is detected.
*/
boolean cLowObstacleCheck()
{
  servoPos(0,90);
  servoPos(1,130);
  delay(200);
  int irCliffCheck = getIrDistanceAvg();
  servoPos(1,90);
  delay(200);
  if(irCliffCheck <= 25 || irCliffCheck >= 42) return true;
  else return false; 
  
}

/*
*  Servo function to simplify the SSC commands
*  servo = 0-7  pos = 0-180
*/
void servoPos(int servo, int pos)
{
  if(servo < 0 || servo > 7 || pos < 0 || pos > 180)
  {
    return;
  }
  int posAdjust = map(pos, 0, 180, SER_LOW_LIMIT, SER_HIGH_LIMIT);
  Serial.print("0P");
  Serial.print(servo);
  Serial.print(" ");
  Serial.println(posAdjust,HEX);
}

void headReset()
{
  servoPos(0,90);
  servoPos(1,90);
  servoPos(2,90);
  servoPos(3,90);
}

/*
*  Return the distance in cm from the sonar
*
*/
int getSonarDistance()
{
  digitalWrite(US_TRIGGER, LOW);
  delayMicroseconds(30);
  digitalWrite(US_TRIGGER, HIGH);
  delayMicroseconds(30);
  digitalWrite(US_TRIGGER, LOW);
  int distance = pulseIn(US_PULSE, HIGH);
    
  distance = (distance/2)/29;
  return distance;
}

/*
*  Fills the sonarRange array with values 
*  10 positions
*/
void fullScanSonar()
{
  headReset();
  lcd.cursorTo(1,1);   
  for(int x = 0; x <= 8; x++)
  {
    servoPos(0,x*20+10);
    if(x == 0) delay(800);
    else delay(150);
    sonarRange[x]= getSonarDistance();
//    delay(100);
//    itoa(sonarRange[x], buffer, 10);
//    lcd.printIn(buffer);
//    lcd.printIn(" ");
//    rfPacketOut('A',sonarRange[x]);
  }
  servoPos(0,90);
}
  
    
/*
*  Return the distance in cm from the sharp IR on irSide.
*  
*/
int getIrDistance(int irSide)
{
  int val = returnAverageAdc(irSide, 2);
  int distance = 0;
  if(val > 3){  distance = (6787 / (val - 3)) - 4; } else{ distance = 80; } //catch divide by zero
  if(distance > 80){ distance = 80; }    //prevent ridiculous numbers coming out from the divide
  return distance;  
}

int getIrDistanceAvg()
{
  int val = getIrDistance(RANGE_IR_PIN_LEFT)+getIrDistance(RANGE_IR_PIN_RIGHT);
  val = val/2;
  return val;
}  

/*
* Return the ADC reading of pinNumber averaged averageCount number of times 
*  Returns 0 if an invalid analog pin is specified
*/
int returnAverageAdc(int pinNumber, int averageCount)
{
  int val = 0;
  if(pinNumber > 4) { return -1; }
  for(int x = 0; x < averageCount; x++)
  {
     val += analogRead(pinNumber);    
  }
  return val/averageCount;
}

/*
*    Some behavioural routines. these are called by the main loop.
*    also some helper functions
*/
void cSearchAndTrack(int durationSeconds)
{
  unsigned long startTime = millis();
  unsigned long durationMillis = durationSeconds*1000;
  int distanceDelta = 20;  // the minimum difference in left right readings to be counted as trackable. in cm
  int lastSide; // the last side eye that saw something. 0 is right, 1 is left
  int bounce = 0;
  int headPos = 90;
  int eyePos = 90;
  int leftIrDistance, rightIrDistance;
  while(startTime+durationMillis >= millis())
  {
    // search with eyes
    servoPos(0, headPos);
    servoPos(3, eyePos);
    delay(10);
    leftIrDistance = getIrDistance(RANGE_IR_PIN_LEFT);
    rightIrDistance = getIrDistance(RANGE_IR_PIN_RIGHT);

    if((leftIrDistance + distanceDelta <= rightIrDistance)&&(leftIrDistance < rightIrDistance))  // left eye sees
    {
      if(headPos <= 5) // limit, so turn body in the direction of the thing
      {
        headPos = 15;
        mGo(SPIN_RIGHT, 800,1);
      }
      else // this prevents oscillating left/right constantly. if we have bounced less than 5 times, then bounce
      {       
        headPos += 1;
        eyePos--;
      }
      if(lastSide == 1) bounce++;  // check if we are just bouncing back and forth.
      else if(lastSide == 0) 
      {
        bounce--;
        lastSide = 0;
      }
    }
    else if((rightIrDistance + distanceDelta <= leftIrDistance)&&(rightIrDistance < leftIrDistance)) // right eye sees
    {
      if(headPos >= 175) 
      { 
        headPos = 165; 
        mGo(SPIN_LEFT, 800,1);
      }
      else
      {
        headPos -= 1;
        eyePos--;
      }
      if(lastSide == 0) bounce++;
      else if(lastSide == 1) 
      {
        bounce--;
        lastSide = 1;
      }
    }
    else if((leftIrDistance < 80)&&(rightIrDistance < 80)) // both see something in range
    {
       eyePos--;
      // lets look up/down
      int rand = random(500);
      twitch(rand);
      
    }
    else // Both eyes see nothing. 80cm
    {
      bounce = 0;
      eyePos++;
    }
    if(eyePos < 60)  eyePos = 60;
    if(eyePos > 110) eyePos = 110;

  }
  headReset();    
}

/*
*  Various "twitches" to add personality
*
*/
void twitch(int twitchNumber)
{
  switch(twitchNumber)
  {
    case 0: servoPos(2,160);    // head tilt 1
            delay(1000);
            servoPos(2,90);
            break;
    case 1: servoPos(2,20);  // head tilt 2
            delay(1000);
            servoPos(2,90);
            break;
    case 2: servoPos(1,110);  // head up
            delay(2000);
            servoPos(1,90);
            break;
    case 3: servoPos(1,70);  // head down
            delay(1500);
            servoPos(1,90);
            break;
    case 4: mGo(CURVE_LEFT, 500,1);// shoot left a bit
            break;
    case 5: mGo(CURVE_RIGHT, 500,1);  // right
            break;
    case 6: mGo(FORWARD, 200,1);  //forwards
            break;
    case 7: mGo(BACKWARD, 200,1);  // back
            break;
    case 8: mGo(FORWARD, 100,1);  // forward, back, forward
            mGo(BACKWARD, 100,1);
            mGo(FORWARD, 100,1);
            break;
    case 9: mGo(CURVE_LEFT, 400,1);  //snake left right left
            mGo(CURVE_RIGHT, 500,1);
            mGo(CURVE_LEFT, 400,1);
            break;
  }

}

/*
*  The robot stays stationary but randomly peers around at stuff
*  just for appearing "smart" and curious.
*/
void cNavelGazing(int durationSeconds)
{
  headReset();
  unsigned long startTime = millis();
  unsigned long durationMillis = durationSeconds*1000;
  int x = 90;
  int y = 90;
  bool done;
  while(startTime+durationMillis >= millis())
  {
    int randX = random(20,161);
    int randY = random(40,120);
    done = 0;
    while(done != 1)
    {
      if(x < randX) x++;
      if(x > randX) x--;
      if(y < randY) y++;
      if(y > randY) y--;
      servoPos(0,x);
      servoPos(1,y);
      if((x == randX) && (y == randY)) done = 1;
      delay(20);
    } 
    delay(4000);
  } 
}

/*
*  Begin following the nearest wall
*  distance = from wall in cms
*  dirClock = 0 = wall on the right. 1 = wall on the left
*/
void cWallFollowing(int durationSeconds, int holdingDistance, int wallSide)
{
  if((holdingDistance < 10)||(wallSide > 1)) return;
  headReset();  
  int maxError = holdingDistance/3;
  boolean wallLocked = false;
  unsigned long startTime = millis();
  unsigned long durationMillis = durationSeconds*1000;
  while(startTime+durationMillis >= millis())
  {
//    lcd.clear();
    int currentDistance = 300;
    int closestAngle = -1;
    // we locked on to a wall and have already turned in the correct direction. Now we have the wall to our left or right.
    // If we angle the eyes at 90 degrees, and the head at 45 degrees to the wall, we can read forward and sideways all at once.
    if(wallLocked == true)  
    {
      servoPos(3, 150);  // angle the eyes 90 degrees to each other
      if(wallSide == 0)  // the wall is on our right, head should be 45 degrees right
      {
        servoPos(0,40); 
        delay(300);
        speedLeft = 254;  // reset the PWMs to normal. They will be changed later if needed
        speedRight = 244;
        int distanceRight = getIrDistance(RANGE_IR_PIN_RIGHT);  // right eye should be pointing at the wall
        int distanceAhead = getIrDistance(RANGE_IR_PIN_LEFT);  // left eye should be pointing ahead
        lcd.clear();
        lcd.cursorTo(1,1);
        itoa(distanceRight, buffer,10);
        lcd.printIn(buffer);
        int errorToWall = distanceRight - holdingDistance;
        lcd.printIn(" ");
        itoa(errorToWall, buffer, 10);
        lcd.printIn(buffer);
        if((distanceAhead - holdingDistance) >= 2)  // continue forwards
        {
          if(errorToWall > maxError)  // way too far from the wall. probably a corner away from the bot.
          {
            mGo(CURVE_RIGHT,1000, 1);
            mGo(FORWARD, 1000, 1);
          }
          else if(errorToWall < (maxError*-1))  // way to close to the wall. something sticking out?
          {
            mGo(CURVE_LEFT, 400, 1);
            mGo(FORWARD, 400, 1);
            mGo(CURVE_RIGHT, 400, 1);
          }
          else if(errorToWall < -1 && errorToWall > (maxError*-1))  // too close to the wall
          {
            speedLeft = 254-(errorToWall*-10);
            speedRight = 244;            
          }
          else if(errorToWall > 1 && errorToWall < maxError) // too far from the wall
          {
            speedLeft = 254;
            speedRight = 244-(errorToWall*10);
          }
          
          analogWrite(MOTOR_PWM_LEFT, speedLeft);
          analogWrite(MOTOR_PWM_RIGHT, speedRight);
          lcd.cursorTo(2,1);
          itoa(speedLeft, buffer, 10);
          lcd.printIn(buffer);
          lcd.printIn(" ");
          itoa(speedRight, buffer, 10);
          lcd.printIn(buffer);

          mGo(FORWARD,0,1);
        }
        else  // prepare for a wall ahead.
        {
          analogWrite(MOTOR_PWM_LEFT, 254);
          analogWrite(MOTOR_PWM_RIGHT, 244);
          mGo(SPIN_LEFT,1200,1);
        }
      }
      else if(wallSide == 1)  // the wall is on our left, head 45 degrees left
      {
        servoPos(0,130);
        delay(300);
        int distanceLeft = getIrDistance(RANGE_IR_PIN_LEFT);  // left eye points at the wall on the left
        int distanceAhead = getIrDistance(RANGE_IR_PIN_RIGHT);  // right eye points straight ahead
        lcd.clear();
        lcd.cursorTo(1,1);
        itoa(distanceLeft, buffer,10);
        lcd.printIn(buffer);
        int errorToWall = distanceLeft - holdingDistance;
        lcd.printIn(" ");
        itoa(errorToWall, buffer, 10);
        lcd.printIn(buffer);
        if((distanceAhead - holdingDistance) >= 2)  // continue forwards
        {
          if(errorToWall > maxError)  // way too far from the wall. probably a corner away from the bot.
          {
//            wallLocked = false;
            mGo(CURVE_LEFT, 1200, 1);
            mGo(FORWARD, 1000, 1);
          }
          if(errorToWall < -1 && errorToWall > (maxError*-1))  // too close to the wall
          {
            speedLeft = 254;
            speedRight = 244-(errorToWall*-10);            
          }
          else if(errorToWall > 1 && errorToWall < maxError) // too far from the wall
          {
            speedLeft = 254-(errorToWall*10);
            speedRight = 244;
          }
          
          analogWrite(MOTOR_PWM_LEFT, speedLeft);
          analogWrite(MOTOR_PWM_RIGHT, speedRight);
          mGo(FORWARD,0,1);
        }
        else  // prepare for a wall ahead.
        {
          analogWrite(MOTOR_PWM_LEFT, speedLeft);
          analogWrite(MOTOR_PWM_RIGHT, speedRight);
          mGo(SPIN_RIGHT,1200,1);
        }
        
      }      
    }
    //to start, find the closest wall. Scan with IR for accuracy.
    if(wallLocked == false)
    {
      mGo(STOP,0,0);
      for(int x = 0; x <= 18; x++)
      {
        servoPos(3,90);
        servoPos(0,x*10);
        if(x == 0) delay(800);
        delay(100);
        int distance = getIrDistanceAvg();
        if(distance < currentDistance)
        {
          closestAngle = x;
          currentDistance = distance;
        }
      }
      if(closestAngle != -1)  // We found the best angle to a wall
      {
        if((closestAngle <= 10)&&(closestAngle >= 8))  // We are now dead on with the wall/obstacle. close to appropriate distance
        {
          lcd.cursorTo(1,1);
          lcd.printIn("dead on");
          servoPos(0,90);
          delay(200);
          while(wallLocked == false)
          {
            currentDistance = getIrDistanceAvg();
            if((holdingDistance - currentDistance) >= 2)  //too close
            {
              mGo(BACKWARD,0,1);
            }
            else if((currentDistance - holdingDistance) >= 2)  //too far
            {
              mGo(FORWARD,0,1);
            }
            else  //within tolerances
            {
              wallLocked = true;   
              if(wallSide == 0) mGo(SPIN_LEFT,1300,1);
              else if(wallSide == 1) mGo(SPIN_RIGHT,1300,1);          
            }
          }
        }      
        else if(closestAngle <= 7)
        {
          mGo(CURVE_RIGHT,(9-closestAngle)*200,1);  // center = 9. 0 = far right.
        }
        else if(closestAngle >= 11)
        {
          mGo(CURVE_LEFT, (closestAngle-9)*200,1);
        }
      }
      else  // saw nothing. Turn around and try again.
      {
        mGo(SPIN_RIGHT, 2000,0);
      }
    }
  }
  
}

/*
*  Begin roaming and avoiding obstacles.
*
*/
void cRoaming(int durationSeconds)
{
  boolean scanAgain = false;
  int rangeMid,rangeLeft,rangeRight =0;
  int distance;
  unsigned long startTime = millis();
  unsigned long durationMillis = durationSeconds*1000;
  headReset();  
  while(startTime+durationMillis >= millis())
  {
    if(cLowObstacleCheck() == true) mGo(SPIN_RIGHT, random(500,1000),0);
    scanAgain == false;
    delay(200);
    rangeMid = getSonarDistance();
    servoPos(0,50);
    delay(500);
    rangeRight = getSonarDistance();
    servoPos(0,130);
    delay(700);
    rangeLeft = getSonarDistance();
    servoPos(0,90);
    if(rangeMid > RANGE_CAUTION)
    {
      mGo(FORWARD,0,0);
    // slowly scan head right to left
    do
    {
    for(int x = 60; x <= 120; x+=2)
    {
      servoPos(0,x);
      delay(30);
      if(getIrDistance(RANGE_IR_PIN_LEFT) < RANGE_CAUTION || getIrDistance(RANGE_IR_PIN_RIGHT) < RANGE_CAUTION)
      {
        if(x < 90)  // object on the right
        {
          mGo(CURVE_LEFT, 0,1);
          for(int y = x; y == 110; y++) 
          {
            delay(30);
            servoPos(0,y);
            
          }
        }
        else if(x > 90)  // object on the left
        {
          mGo(CURVE_RIGHT, 0, 1);
          for(int y = x; y == 70; y--)
          {
            delay(30);
            servoPos(0,y);
          }
        }
        else if(x == 90) mGo(SPIN_RIGHT, random(1000,1500),1);
        scanAgain = true;
        mGo(STOP,0,0);
        break;
      }
    }
    for(int x = 120; x >= 60; x-=2)
    {
      servoPos(0,x);
      delay(30);
      if(getIrDistance(RANGE_IR_PIN_LEFT) < RANGE_CAUTION || getIrDistance(RANGE_IR_PIN_RIGHT) < RANGE_CAUTION)
      {
        if(x < 90)  // object on the right
        {
          mGo(CURVE_LEFT, 0,1);
          for(; x == 110; x++) 
          {
            delay(30);
            servoPos(0,x);
          }
        }
        else if(x > 90)
        {
          mGo(CURVE_RIGHT, 0, 1);
          for(; x == 70; x--)
          {
            delay(30);
            servoPos(0,x);
          }
        }
        else if(x == 90) mGo(SPIN_LEFT, random(1000,1500),1);
        scanAgain = true;
        mGo(STOP,0,0);
        break;
      }      
    }
    
    }    while(scanAgain == false);

    delay(1000);
    }
    else
    {
      mGo(SPIN_LEFT,random(500,1500),0);
    }
    
  }
  mGo(STOP,0,0);      
}

/*
*  If val = 0 it will print the time and date in a standard format taking up the whole LCD
*  If a number 1-7 is passed, it will read and return that number without printing it
*  Numbers go from 1= second to 7= year.
*/
int updateTimeDate(int val)
{
  byte second, minute, hour, dayOfMonth, dayOfWeek, month, year;
  getDateRTC(&second, &minute, &hour, &dayOfMonth, &dayOfWeek, &month, &year);
  if(val == 0)
  {
    lcd.cursorTo(1,7);
    itoa(hour, buffer, 10);
    if(hour < 10){lcd.printIn("0");}  // 0 space
    lcd.printIn(buffer);
    lcd.printIn(":");
    itoa(minute, buffer, 10);
    if(minute < 10){lcd.printIn("0");}
    lcd.printIn(buffer);
    lcd.cursorTo(2,3);
    switch(dayOfWeek)
    {
    case 0: lcd.printIn("Saturday");
            break;
    case 1: lcd.printIn("Sunday");
            break;
    case 2: lcd.printIn("Monday");
            break;
    case 3: lcd.printIn("Tuesday");
          break;
    case 4: lcd.printIn("Wednesday");
            break;
    case 5: lcd.printIn("Thursday");
            break;
    case 6: lcd.printIn("Friday");
    }
    lcd.printIn(" ");
    itoa(dayOfMonth, buffer, 10);
    lcd.printIn(buffer);
    lcd.printIn("-");
    itoa(month, buffer, 10);
    lcd.printIn(buffer);
    lcd.printIn("  ");
    return -1;
  }
  else
  {
    switch(val)
    {
      case 1: return second;
              break;
      case 2: return minute;
              break;
      case 3: return hour;
              break;
      case 4: return dayOfMonth;
              break;
      case 5: return dayOfWeek;
              break;
      case 6: return month;
              break;
      case 7: return year;
              break;        
    }     
  } 
  return -1;
    
}

void setup()
{
  pinMode(MOTOR_ENC_LEFT, INPUT);
  pinMode(MOTOR_ENC_RIGHT, INPUT);
  pinMode(MOTOR_PWM_LEFT, OUTPUT);
  pinMode(MOTOR_PWM_RIGHT, OUTPUT);
  pinMode(US_TRIGGER, OUTPUT);
  pinMode(US_PULSE, INPUT);

  Wire.begin();		    // join i2c bus with address #4
  Serial.begin(9600);  // serial port to the servo controller

  Serial.println("0CP 00");  // set pwm to servo mode
  delay(10);
  Serial.println("0CD FF");
  // center all servos
  servoPos(0,90);
  servoPos(1,90);
  servoPos(2,90);
  servoPos(3,90);
  servoPos(4,90);
  servoPos(5,90);
  servoPos(6,90);
  servoPos(7,90);
  lcd.init();
  lcd.backLight(true);
  lcd.setCustomCharacter(1, lcdSmile); // load custom characters
  lcd.setCustomCharacter(2, lcdLeft);
  lcd.setCustomCharacter(3, lcdRight);
  lcd.setCustomCharacter(4, lcdReturn);
  lcd.setCustomCharacter(5, lcdBack);
  
  analogWrite(MOTOR_PWM_RIGHT,speedRight);
  analogWrite(MOTOR_PWM_LEFT,speedLeft);
  lcd.cursorTo(1,1);
  int hour = updateTimeDate(3); 
  if((hour >= 6)&&(hour <= 11)) { lcd.printIn("  Good morning  "); }
  else if((hour >= 12)&&(hour <= 17)) { lcd.printIn(" Good afternoon "); }
  else if((hour >= 18)&&(hour <= 23)) { lcd.printIn("  Good evening  "); }
  else { lcd.printIn("     Hello     "); }
  delay(2000);
  lcd.clear();
  vw_set_tx_pin(9);
  vw_setup(2400);
  attachInterrupt(0,encLeft,FALLING);
  attachInterrupt(1,encRight,FALLING);
  
}

void loop()
{
  randomSeed(millis());
//  cNavelGazing(30);  // done
//  cRoaming(50);
//  cSearchAndTrack(50);  // done well enough
//  cWallFollowing(40,25,0);  // works ok, overcorrects.
//  fullScanSonar();  // ok
//  rfPacketOut('A', 134); // ok.

}
